@@ -7,16 +7,16 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@
 
 #include <gmodule.h>
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-splash-engine.h>
 #include <libxfsm/xfsm-splash-rc.h>
@@ -183,7 +183,7 @@ void
 module_test (Module     *module,
              GdkDisplay *display)
 {
-  static char *steps[] = 
+  static char *steps[] =
   {
     "Starting the Window Manager",
     "Starting the Desktop Manager",
@@ -203,8 +203,8 @@ module_test (Module     *module,
   bzero (&engine, sizeof (engine));
 
   /* locate monitor with pointer */
-  screen = xfce_gdk_display_locate_monitor_with_pointer (display, &monitor);
-  if (G_UNLIKELY (screen == NULL))
+  screen = xfce_gdk_screen_get_active (&monitor);
+  if (G_UNLIKELY (screen == NULL) || (gdk_screen_get_display(screen) != display))
     {
       screen  = gdk_display_get_screen (display, 0);
       monitor = 0;
