@@ -7,22 +7,22 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifndef __XFAE_DIALOG_H__
 #define __XFAE_DIALOG_H__
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS;
 
@@ -38,7 +38,9 @@ typedef struct _XfaeDialog      XfaeDialog;
 
 GType      xfae_dialog_get_type (void) G_GNUC_CONST;
 
-GtkWidget *xfae_dialog_new      (void);
+GtkWidget *xfae_dialog_new      (const gchar *name,
+                                 const gchar *descr,
+                                 const gchar *command);
 
 void       xfae_dialog_get      (XfaeDialog *dialog,
                                  gchar     **name,
