@@ -8,16 +8,16 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,20 +55,30 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-util.h>
 
 #include <xfce4-session/ice-layer.h>
-#include <xfce4-session/shutdown.h>
 #include <xfce4-session/sm-layer.h>
 #include <xfce4-session/xfsm-dns.h>
 #include <xfce4-session/xfsm-global.h>
 #include <xfce4-session/xfsm-manager.h>
+#include <xfce4-session/xfsm-shutdown.h>
 #include <xfce4-session/xfsm-startup.h>
-#include "xfsm-error.h"
+#include <xfce4-session/xfsm-error.h>
 
-void
+static gboolean opt_disable_tcp = FALSE;
+static gboolean opt_version = FALSE;
+
+static GOptionEntry option_entries[] =
+{
+  { "disable-tcp", '\0', 0, G_OPTION_ARG_NONE, &opt_disable_tcp, N_("Disable binding to TCP ports"), NULL },
+  { "version", 'V', 0, G_OPTION_ARG_NONE, &opt_version, N_("Print version information and exit"), NULL },
+  { NULL }
+};
+
+static void
 setup_environment (void)
 {
   const gchar *lang;
@@ -76,11 +86,11 @@ setup_environment (void)
   gchar       *authfile;
   int          fd;
 
-  /* check that no other session manager is running */  
+  /* check that no other session manager is running */
   sm = g_getenv ("SESSION_MANAGER");
   if (sm != NULL && strlen (sm) > 0)
     {
-      fprintf (stderr, "xfce4-session: Another session manager is already running\n");
+      g_printerr ("%s: Another session manager is already running\n", PACKAGE_NAME);
       exit (EXIT_FAILURE);
     }
 
@@ -93,7 +103,7 @@ setup_environment (void)
 
   /* this is for compatibility with the GNOME Display Manager */
   lang = g_getenv ("GDM_LANG");
-  if (lang != NULL)
+  if (lang != NULL && strlen (lang) > 0)
     {
       xfce_setenv ("LANG", lang, TRUE);
       xfce_unsetenv ("GDM_LANG");
@@ -115,25 +125,6 @@ setup_environment (void)
   close (fd);
 }
 
-
-static void
-usage (int exit_code)
-{
-  fprintf (stderr,
-           "Usage: xfce4-session [OPTION...]\n"
-           "\n"
-           "GTK+\n"
-           "  --display=DISPLAY        X display to use\n"
-           "\n"
-           "Application options\n"
-           "  --disable-tcp            Disable binding to TCP ports\n"
-           "  --help                   Print this help message and exit\n"
-           "  --version                Print version information and exit\n"
-           "\n");
-  exit (exit_code);
-}
-
-
 static void
 init_display (XfsmManager   *manager,
               GdkDisplay    *dpy,
@@ -144,7 +135,7 @@ init_display (XfsmManager   *manager,
 
   engine = xfconf_channel_get_string (channel, "/splash/Engine", "mice");
 
-  splash_screen = xfsm_splash_screen_new (dpy, engine);  
+  splash_screen = xfsm_splash_screen_new (dpy, engine);
   g_free (engine);
   xfsm_splash_screen_next (splash_screen, _("Loading desktop settings"));
 
@@ -152,72 +143,12 @@ init_display (XfsmManager   *manager,
 
   sm_init (channel, disable_tcp, manager);
 
-  /* start xfsettingsd */
-  if ( !g_spawn_command_line_async ("xfsettingsd", NULL))
-  {
-    g_warning ("Could not start xfsettingsd");
-  }
-  
-  
   /* gtk resource files may have changed */
   gtk_rc_reparse_all ();
 }
 
 
 static void
-initialize (XfsmManager *manager,
-            int          argc,
-            char       **argv)
-{
-  gboolean disable_tcp = FALSE;
-  GdkDisplay *dpy;
-  XfconfChannel *channel;
-  
-  for (++argv; --argc > 0; ++argv)
-    {
-      if (strcmp (*argv, "--version") == 0)
-        {
-          printf ("%s (Xfce %s)\n\n"
-                  "Copyright (c) 2003-2006\n"
-                  "        The Xfce development team. All rights reserved.\n\n"
-                  "Written for Xfce by Benedikt Meurer <benny@xfce.org>.\n\n"
-                  "Built with Gtk+-%d.%d.%d, running with Gtk+-%d.%d.%d.\n\n"
-                  "Please report bugs to <%s>.\n",
-                  PACKAGE_STRING, xfce_version_string (),
-                  GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
-                  gtk_major_version, gtk_minor_version, gtk_micro_version,
-                  PACKAGE_BUGREPORT);
-          exit (EXIT_SUCCESS);
-        }
-      else if (strcmp (*argv, "--disable-tcp") == 0)
-        {
-          disable_tcp = TRUE;
-        }
-      else
-        {
-          usage (strcmp (*argv, "--help") == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
-        }
-    }
-
-  setup_environment ();
-
-  channel = xfsm_open_config ();
-
-  dpy = gdk_display_get_default ();
-  init_display (manager, dpy, channel, disable_tcp);
-
-  /* verify that the DNS settings are ok */
-  xfsm_splash_screen_next (splash_screen, _("Verifying DNS settings"));
-  xfsm_dns_check ();
-
-  xfsm_splash_screen_next (splash_screen, _("Loading session data"));
-
-  xfsm_startup_init (channel);
-  xfsm_manager_load (manager, channel);
-}
-
-
-void
 xfsm_dbus_init (void)
 {
   DBusGConnection *dbus_conn;
@@ -241,12 +172,12 @@ xfsm_dbus_init (void)
                                NULL);
   if (DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER != ret)
     {
-      g_error ("Another session manager is already running");
-      exit (1);
+      g_printerr ("%s: Another session manager is already running\n", PACKAGE_NAME);
+      exit (EXIT_FAILURE);
     }
 }
 
-void
+static void
 xfsm_dbus_cleanup (void)
 {
   DBusGConnection *dbus_conn;
@@ -261,29 +192,90 @@ xfsm_dbus_cleanup (void)
                          "org.xfce.SessionManager", NULL);
 }
 
+static gboolean
+xfsm_dbus_require_session (gint argc, gchar **argv)
+{
+  gchar **new_argv;
+  gchar  *path;
+  gint    i;
+  guint   m = 0;
+
+  if (g_getenv ("DBUS_SESSION_BUS_ADDRESS") != NULL)
+    return TRUE;
+
+  path = g_find_program_in_path ("dbus-launch");
+  if (path == NULL)
+    {
+      g_critical ("dbus-launch not found, the desktop will not work properly!");
+      return TRUE;
+    }
+
+  /* avoid rondtrips */
+  g_assert (!g_str_has_prefix (*argv, "dbus-launch"));
+
+  new_argv = g_new0 (gchar *, argc + 4);
+  new_argv[m++] = path;
+  new_argv[m++] = "--sh-syntax";
+  new_argv[m++] = "--exit-with-session";
+
+  for (i = 0; i < argc; i++)
+    new_argv[m++] = argv[i];
+
+  if (!execvp ("dbus-launch", new_argv))
+    {
+      g_critical ("Could not spawn %s: %s", path, g_strerror (errno));
+    }
+
+  g_free (path);
+  g_free (new_argv);
+
+  return FALSE;
+}
+
 int
 main (int argc, char **argv)
 {
-  XfsmManager     *manager;
-  XfsmShutdownType shutdown_type;
-  GError          *error = NULL;
+  XfsmManager      *manager;
+  GError           *error = NULL;
+  GdkDisplay       *dpy;
+  XfconfChannel    *channel;
+  XfsmShutdownType  shutdown_type;
+  XfsmShutdown     *shutdown_helper;
+  gboolean          succeed = TRUE;
+
+  if (!xfsm_dbus_require_session (argc, argv))
+    return EXIT_SUCCESS;
 
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-  
+
   /* install required signal handlers */
   signal (SIGPIPE, SIG_IGN);
 
-  gtk_init (&argc, &argv);
+  if (!gtk_init_with_args (&argc, &argv, "", option_entries, GETTEXT_PACKAGE, &error))
+    {
+      g_print ("%s: %s.\n", G_LOG_DOMAIN, error->message);
+      g_print (_("Type '%s --help' for usage."), G_LOG_DOMAIN);
+      g_print ("\n");
+      g_error_free (error);
+      return EXIT_FAILURE;
+    }
 
-  if (G_UNLIKELY (!xfconf_init (&error))) {
-    xfce_message_dialog (NULL, _("Xfce Session Manager"),
-                         GTK_STOCK_DIALOG_ERROR,
-                         _("Unable to contact settings server"),
-                         error->message,
-                         GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
-                         NULL);
-    g_error_free (error);
-  }
+  if (opt_version)
+    {
+      g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
+      g_print ("%s\n", "Copyright (c) 2003-2012");
+      g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
+      g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
+      g_print ("\n");
+
+      return EXIT_SUCCESS;
+    }
+
+  if (!xfconf_init (&error))
+    {
+      xfce_dialog_show_error (NULL, error, _("Unable to contact settings server"));
+      g_error_free (error);
+    }
 
   /* fake a client id for the manager, so the legacy management does not
    * recognize us to be a session client.
@@ -293,16 +285,50 @@ main (int argc, char **argv)
   xfsm_dbus_init ();
 
   manager = xfsm_manager_new ();
-  initialize (manager, argc, argv);
+  setup_environment ();
+
+  channel = xfsm_open_config ();
+
+  dpy = gdk_display_get_default ();
+  init_display (manager, dpy, channel, opt_disable_tcp);
+
+  if (!opt_disable_tcp && xfconf_channel_get_bool (channel, "/security/EnableTcp", FALSE))
+    {
+      /* verify that the DNS settings are ok */
+      xfsm_splash_screen_next (splash_screen, _("Verifying DNS settings"));
+      xfsm_dns_check ();
+    }
+
+  xfsm_splash_screen_next (splash_screen, _("Loading session data"));
+
+  xfsm_startup_init (channel);
+  xfsm_manager_load (manager, channel);
   xfsm_manager_restart (manager);
-  
+
   gtk_main ();
 
+  xfsm_startup_shutdown ();
+
   shutdown_type = xfsm_manager_get_shutdown_type (manager);
+
+  /* take over the ref before we release the manager */
+  shutdown_helper = xfsm_shutdown_get ();
+
   g_object_unref (manager);
- 
+  g_object_unref (channel);
+
   xfsm_dbus_cleanup ();
   ice_cleanup ();
 
-  return xfsm_shutdown (shutdown_type);
+  if (shutdown_type == XFSM_SHUTDOWN_SHUTDOWN
+      || shutdown_type == XFSM_SHUTDOWN_RESTART)
+    {
+      succeed = xfsm_shutdown_try_type (shutdown_helper, shutdown_type, &error);
+      if (!succeed)
+        g_warning ("Failed to shutdown/restart: %s", ERROR_MSG (error));
+    }
+
+  g_object_unref (shutdown_helper);
+
+  return succeed ? EXIT_SUCCESS : EXIT_FAILURE;
 }
