@@ -7,16 +7,16 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,10 @@
 
 #define XFSM_CLIENT_MANAGER(c)  (XFSM_MANAGER (g_object_get_data (G_OBJECT (c), "--xfsm-manager")))
 
+#ifdef HAVE__ICETRANSNOLISTEN
+extern void _IceTransNoListen (char *protocol);
+#endif
+
 /* local prototypes */
 static Status sm_new_client                 (SmsConn         sms_conn,
                                              SmPointer       manager_data,
@@ -108,7 +112,6 @@ sm_init (XfconfChannel *channel,
   if (disable_tcp || !xfconf_channel_get_bool (channel, "/security/EnableTcp", FALSE))
     {
 #ifdef HAVE__ICETRANSNOLISTEN
-      extern void _IceTransNoListen (char *protocol);
       _IceTransNoListen ("tcp");
 #else
       if (G_UNLIKELY (verbose))
@@ -120,7 +123,7 @@ sm_init (XfconfChannel *channel,
         }
 #endif
     }
-  
+
   if (!SmsInitialize (PACKAGE, VERSION, sm_new_client, manager, ice_auth_proc,
                       2048, error))
     {
@@ -133,9 +136,9 @@ sm_init (XfconfChannel *channel,
       fprintf (stderr, "xfce4-session: Unable to establish ICE listeners: %s\n", error);
       exit (EXIT_FAILURE);
     }
-  
+
   ice_setup_listeners (num_listeners, listen_objs, manager);
-  
+
   network_idlist = IceComposeNetworkIdList (num_listeners, listen_objs);
   xfce_setenv ("SESSION_MANAGER", network_idlist, TRUE);
   free (network_idlist);
@@ -155,7 +158,7 @@ sm_new_client (SmsConn        sms_conn,
 
   xfsm_verbose ("ICE connection fd = %d, received NEW CLIENT\n\n",
                 IceConnectionNumber (SmsGetIceConnection (sms_conn)));
-  
+
   client = xfsm_manager_new_client (manager, sms_conn, &error);
   if (client == NULL)
     {
@@ -171,7 +174,7 @@ sm_new_client (SmsConn        sms_conn,
 
       return False;
     }
-  
+
   SET_CALLBACK (callbacks, register_client, client);
   SET_CALLBACK (callbacks, interact_request, client);
   SET_CALLBACK (callbacks, interact_done, client);
@@ -182,15 +185,15 @@ sm_new_client (SmsConn        sms_conn,
   SET_CALLBACK (callbacks, set_properties, client);
   SET_CALLBACK (callbacks, delete_properties, client);
   SET_CALLBACK (callbacks, get_properties, client);
-  
+
   *mask = SmsRegisterClientProcMask | SmsInteractRequestProcMask
-    | SmsInteractDoneProcMask | SmsSaveYourselfRequestProcMask 
+    | SmsInteractDoneProcMask | SmsSaveYourselfRequestProcMask
     | SmsSaveYourselfP2RequestProcMask | SmsSaveYourselfDoneProcMask
     | SmsCloseConnectionProcMask | SmsSetPropertiesProcMask
     | SmsDeletePropertiesProcMask | SmsGetPropertiesProcMask;
 
   g_object_set_data (G_OBJECT (client), "--xfsm-manager", manager);
-                                                                                
+
   return True;
 }
 
@@ -202,16 +205,16 @@ sm_register_client (SmsConn   sms_conn,
 {
   XfsmClient *client = (XfsmClient *) client_data;
   Status      result;
-  
+
   xfsm_verbose ("ICE connection fd = %d, received REGISTER CLIENT [Previous Id = %s]\n\n",
                 IceConnectionNumber (SmsGetIceConnection (sms_conn)),
                 previous_id != NULL ? previous_id : "None");
-  
+
   result = xfsm_manager_register_client (XFSM_CLIENT_MANAGER (client), client, previous_id);
 
   if (previous_id != NULL)
     free (previous_id);
-  
+
   return result;
 }
 
@@ -225,7 +228,7 @@ sm_interact_request (SmsConn   sms_conn,
 
   xfsm_verbose ("Client Id = %s, received INTERACT REQUEST [Dialog type = %s]\n\n",
                 xfsm_client_get_id (client), dialog_type == SmDialogError ? "Error" : "Normal");
-  
+
   xfsm_manager_interact (XFSM_CLIENT_MANAGER (client), client, dialog_type);
 }
 
@@ -239,7 +242,7 @@ sm_interact_done (SmsConn   sms_conn,
 
   xfsm_verbose ("Client Id = %s, received INTERACT DONE [Cancel shutdown = %s]\n\n",
                 xfsm_client_get_id (client), cancel_shutdown ? "True" : "False");
-  
+
   xfsm_manager_interact_done (XFSM_CLIENT_MANAGER (client), client, cancel_shutdown);
 }
 
@@ -263,14 +266,14 @@ sm_save_yourself_request (SmsConn   sms_conn,
                     save_type == SmSaveLocal ? "Local"
                     : (save_type == SmSaveGlobal ? "Global" : "Both"));
       xfsm_verbose ("   Shutdown:       %s\n", shutdown ? "True" : "False");
-      xfsm_verbose ("   Interact Style: %s\n", 
+      xfsm_verbose ("   Interact Style: %s\n",
                     interact_style == SmInteractStyleNone ? "None"
                     : (interact_style == SmInteractStyleErrors ? "Errors" : "Any"));
       xfsm_verbose ("   Fast:           %s\n", fast ? "True" : "False");
       xfsm_verbose ("   Global:         %s\n", global ? "True" : "False");
       xfsm_verbose ("\n");
     }
-  
+
   xfsm_manager_save_yourself (XFSM_CLIENT_MANAGER (client), client, save_type, shutdown, interact_style, fast, global);
 }
 
@@ -283,7 +286,7 @@ sm_save_yourself_phase2_request (SmsConn   sms_conn,
 
   xfsm_verbose ("Client Id = %s, received SAVE YOURSELF PHASE2 REQUEST\n\n",
                 xfsm_client_get_id (client));
-  
+
   xfsm_manager_save_yourself_phase2 (XFSM_CLIENT_MANAGER (client), client);
 }
 
@@ -297,7 +300,7 @@ sm_save_yourself_done (SmsConn   sms_conn,
 
   xfsm_verbose ("Client Id = %s, received SAVE YOURSELF DONE [Success = %s]\n\n",
                 xfsm_client_get_id (client), success ? "True" : "False");
-  
+
   xfsm_manager_save_yourself_done (XFSM_CLIENT_MANAGER (client), client, success);
 }
 
@@ -319,7 +322,7 @@ sm_close_connection (SmsConn   sms_conn,
         xfsm_verbose ("   Reason %2d: %s\n", n, reasons[n]);
       xfsm_verbose ("\n");
     }
-  
+
   xfsm_manager_close_connection (XFSM_CLIENT_MANAGER (client), client, TRUE);
 
   if (num_reasons > 0)
@@ -336,14 +339,15 @@ sm_set_properties (SmsConn   sms_conn,
   XfsmClient *client = (XfsmClient *) client_data;
   int         n;
   int         i;
-  
+
   if (G_UNLIKELY (verbose))
     {
       xfsm_verbose ("Client Id = %s, received SET PROPERTIES [Num props = %d]\n",
                     xfsm_client_get_id (client), num_props);
       for (n = 0; n < num_props; ++n)
         {
-          xfsm_verbose ("   Name:  %s\n   Type:  %s\n", props[n]->name, props[n]->type);
+          xfsm_verbose ("   Name:  %s\n", props[n]->name);
+          xfsm_verbose ("   Type:  %s\n", props[n]->type);
           if (strcmp (props[n]->type, "ARRAY8") == 0)
             {
               xfsm_verbose ("   Value: %s\n", (const gchar *) props[n]->vals->value);
@@ -355,19 +359,20 @@ sm_set_properties (SmsConn   sms_conn,
             }
           else if (strcmp (props[n]->type, "LISTofARRAY8") == 0)
             {
-              xfsm_verbose ("   Value: ");
+              xfsm_verbose ("   Value:\n");
               for (i = 0; i < props[n]->num_vals; ++i)
                 {
-                  xfsm_verbose ("%s%s", (const gchar *) props[n]->vals[i].value,
+                  xfsm_verbose ("          %s%s\n", (const gchar *) props[n]->vals[i].value,
                                 (i == props[n]->num_vals - 1) ? ""  : ",");
                 }
             }
+          xfsm_verbose ("\n");
         }
       xfsm_verbose ("\n");
-    }  
-  
+    }
+
   xfsm_client_merge_properties (client, props, num_props);
-  
+
   while (num_props-- > 0)
     SmFreeProperty (props[num_props]);
   free (props);
@@ -391,9 +396,9 @@ sm_delete_properties (SmsConn   sms_conn,
         xfsm_verbose ("   Name:   %s\n", prop_names[n]);
       xfsm_verbose ("\n");
     }
-  
+
   xfsm_client_delete_properties (client, prop_names, num_props);
-  
+
   while (num_props-- > 0)
     free (prop_names[num_props]);
   free (prop_names);
@@ -410,11 +415,11 @@ sm_get_properties (SmsConn   sms_conn,
   gint            num_props  = 0;
 
   xfsm_verbose ("Client Id = %s, received GET PROPERTIES\n\n", properties->client_id);
-  
+
   xfsm_properties_extract (properties, &num_props, &props);
 
   SmsReturnProperties (sms_conn, num_props, props);
-  
+
   if (num_props > 0)
     {
       while (num_props-- > 0)
