@@ -1,26 +1,24 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
+ * Copyright (c) 2010 Jannis Pohlmann <jannis@xfce.org>
  * All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *                                                                              
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
  *
- * The session id generator was taken from the KDE session manager.
- * Copyright (c) 2000 Matthias Ettrich <ettrich@kde.org>
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -73,20 +71,20 @@
 
 #include <libwnck/libwnck.h>
 
-#include "xfsm-shutdown-helper.h"
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-splash-engine.h>
 #include <libxfsm/xfsm-util.h>
 
-#include "xfsm-manager.h"
-#include "chooser-icon.h"
-#include "xfsm-chooser.h"
-#include "xfsm-global.h"
-#include "xfsm-legacy.h"
-#include "xfsm-startup.h"
-#include "xfsm-marshal.h"
-#include "xfsm-error.h"
+#include <xfce4-session/xfsm-manager.h>
+#include <xfce4-session/xfsm-chooser-icon.h>
+#include <xfce4-session/xfsm-chooser.h>
+#include <xfce4-session/xfsm-global.h>
+#include <xfce4-session/xfsm-legacy.h>
+#include <xfce4-session/xfsm-startup.h>
+#include <xfce4-session/xfsm-marshal.h>
+#include <xfce4-session/xfsm-error.h>
+#include <xfce4-session/xfsm-logout-dialog.h>
 
 
 #define DEFAULT_SESSION_NAME "Default"
@@ -97,13 +95,17 @@ struct _XfsmManager
   GObject parent;
 
   XfsmManagerState state;
-  XfsmShutdownType shutdown_type;
+
+  XfsmShutdownType  shutdown_type;
+  XfsmShutdown     *shutdown_helper;
 
   gboolean         session_chooser;
   gchar           *session_name;
   gchar           *session_file;
   gchar           *checkpoint_session_name;
 
+  gboolean         start_at;
+
   gboolean         compat_gnome;
   gboolean         compat_kde;
 
@@ -117,7 +119,7 @@ struct _XfsmManager
 
   guint            die_timeout_id;
 
-  DBusGConnection *dbus_conn;
+  DBusGConnection *session_bus;
 };
 
 typedef struct _XfsmManagerClass
@@ -158,8 +160,6 @@ enum
 };
 
 
-static void       xfsm_manager_class_init (XfsmManagerClass *klass);
-static void       xfsm_manager_init (XfsmManager *manager);
 static void       xfsm_manager_finalize (GObject *obj);
 
 static gboolean   xfsm_manager_startup (XfsmManager *manager);
@@ -229,6 +229,7 @@ xfsm_manager_init (XfsmManager *manager)
   manager->session_chooser = FALSE;
   manager->failsafe_mode = TRUE;
   manager->shutdown_type = XFSM_SHUTDOWN_LOGOUT;
+  manager->shutdown_helper = xfsm_shutdown_get ();
 
   manager->pending_properties = g_queue_new ();
   manager->starting_properties = g_queue_new ();
@@ -237,7 +238,6 @@ xfsm_manager_init (XfsmManager *manager)
   manager->failsafe_clients = g_queue_new ();
 }
 
-
 static void
 xfsm_manager_finalize (GObject *obj)
 {
@@ -248,6 +248,8 @@ xfsm_manager_finalize (GObject *obj)
   if (manager->die_timeout_id != 0)
     g_source_remove (manager->die_timeout_id);
 
+  g_object_unref (manager->shutdown_helper);
+
   g_queue_foreach (manager->pending_properties, (GFunc) xfsm_properties_free, NULL);
   g_queue_free (manager->pending_properties);
 
@@ -349,6 +351,9 @@ gboolean
 xfsm_manager_handle_failed_properties (XfsmManager    *manager,
                                        XfsmProperties *properties)
 {
+  gint restart_style_hint;
+  GError *error = NULL;
+
   /* Handle apps that failed to start, or died randomly, here */
 
   xfsm_properties_set_default_child_watch (properties);
@@ -359,11 +364,15 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
       properties->restart_attempts_reset_id = 0;
     }
 
-  if (properties->restart_style_hint == SmRestartAnyway)
+  restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                  SmRestartStyleHint,
+                                                  SmRestartIfRunning);
+
+  if (restart_style_hint == SmRestartAnyway)
     {
       g_queue_push_tail (manager->restart_properties, properties);
     }
-  else if (properties->restart_style_hint == SmRestartImmediately)
+  else if (restart_style_hint == SmRestartImmediately)
     {
       if (++properties->restart_attempts > MAX_RESTART_ATTEMPTS)
         {
@@ -393,6 +402,8 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
     }
   else
     {
+      gchar **discard_command;
+
       /* We get here if a SmRestartNever or SmRestartIfRunning client
        * has exited.  SmRestartNever clients shouldn't have discard
        * commands, but it can't hurt to run it if it has one for some
@@ -400,7 +411,8 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
       xfsm_verbose ("Client Id %s exited, removing from session.\n",
                     properties->client_id);
 
-      if (properties->discard_command != NULL)
+      discard_command = xfsm_properties_get_strv (properties, SmDiscardCommand);
+      if (discard_command != NULL && g_strv_length (discard_command) > 0)
         {
           /* Run the SmDiscardCommand after the client exited in any state,
            * but only if we don't expect the client to be restarted,
@@ -419,16 +431,22 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
            * ever-growing number of xfwm4 session files when restarting
            * xfwm4 within a session.
            */
-          xfsm_verbose ("Client Id = %s: running discard command.\n\n",
-                        properties->client_id);
+          xfsm_verbose ("Client Id = %s: running discard command %s:%d.\n\n",
+                        properties->client_id, *discard_command,
+                        g_strv_length (discard_command));
 
-          g_spawn_sync (properties->current_directory,
-                        properties->discard_command,
-                        properties->environment,
-                        G_SPAWN_SEARCH_PATH,
-                        NULL, NULL,
-                        NULL, NULL,
-                        NULL, NULL);
+          if (!g_spawn_sync (xfsm_properties_get_string (properties, SmCurrentDirectory),
+                             discard_command,
+                             xfsm_properties_get_strv (properties, SmEnvironment),
+                             G_SPAWN_SEARCH_PATH,
+                             NULL, NULL,
+                             NULL, NULL,
+                             NULL, &error))
+            {
+              g_warning ("Failed to running discard command \"%s\": %s",
+                         *discard_command, error->message);
+              g_error_free (error);
+            }
         }
 
       return FALSE;
@@ -467,8 +485,8 @@ xfsm_manager_choose_session (XfsmManager *manager,
             {
               if (G_UNLIKELY (preview_default == NULL))
                 {
-                  preview_default = gdk_pixbuf_from_pixdata (&chooser_icon_data,
-                                                             FALSE, NULL);
+                  preview_default = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data,
+                                                                FALSE, NULL);
                 }
 
               session->preview = GDK_PIXBUF (g_object_ref (preview_default));
@@ -523,14 +541,14 @@ xfsm_manager_load_session (XfsmManager *manager)
   gchar           buffer[1024];
   XfceRc         *rc;
   gint            count;
-  
+
   if (!g_file_test (manager->session_file, G_FILE_TEST_IS_REGULAR))
     return FALSE;
 
   rc = xfce_rc_simple_open (manager->session_file, FALSE);
   if (G_UNLIKELY (rc == NULL))
     return FALSE;
-  
+
   if (manager->session_chooser && !xfsm_manager_choose_session (manager, rc))
     {
       xfce_rc_close (rc);
@@ -538,7 +556,7 @@ xfsm_manager_load_session (XfsmManager *manager)
     }
 
   g_snprintf (buffer, 1024, "Session: %s", manager->session_name);
-  
+
   xfce_rc_set_group (rc, buffer);
   count = xfce_rc_read_int_entry (rc, "Count", 0);
   if (G_UNLIKELY (count <= 0))
@@ -546,7 +564,7 @@ xfsm_manager_load_session (XfsmManager *manager)
       xfce_rc_close (rc);
       return FALSE;
     }
-  
+
   while (count-- > 0)
     {
       g_snprintf (buffer, 1024, "Client%d_", count);
@@ -659,7 +677,7 @@ xfsm_manager_load_settings (XfsmManager   *manager,
                             XfconfChannel *channel)
 {
   gboolean session_loaded = FALSE;
-  
+
   manager->session_name = xfconf_channel_get_string (channel,
                                                      "/general/SessionName",
                                                      DEFAULT_SESSION_NAME);
@@ -718,8 +736,9 @@ xfsm_manager_load (XfsmManager   *manager,
 
   manager->compat_gnome = xfconf_channel_get_bool (channel, "/compat/LaunchGNOME", FALSE);
   manager->compat_kde = xfconf_channel_get_bool (channel, "/compat/LaunchKDE", FALSE);
+  manager->start_at = xfconf_channel_get_bool (channel, "/general/StartAssistiveTechnologies", FALSE);
 
-  display_name  = xfce_gdk_display_get_fullname (gdk_display_get_default ());
+  display_name  = xfsm_gdk_display_get_fullname (gdk_display_get_default ());
 
 #ifdef HAVE_OS_CYGWIN
   /* rename a colon (:) to a hash (#) under cygwin. windows doesn't like
@@ -752,13 +771,13 @@ xfsm_manager_restart (XfsmManager *manager)
   /* tell splash screen that the session is starting now */
   preview = xfsm_load_session_preview (manager->session_name);
   if (preview == NULL)
-    preview = gdk_pixbuf_from_pixdata (&chooser_icon_data, FALSE, NULL);
+    preview = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data, FALSE, NULL);
   steps = g_queue_get_length (manager->failsafe_mode ? manager->failsafe_clients : manager->pending_properties);
   xfsm_splash_screen_start (splash_screen, manager->session_name, preview, steps);
   g_object_unref (preview);
 
   g_idle_add ((GSourceFunc) xfsm_manager_startup, manager);
-  
+
   return TRUE;
 }
 
@@ -902,7 +921,7 @@ xfsm_manager_register_client (XfsmManager *manager,
 
   g_signal_emit (manager, signals[SIG_CLIENT_REGISTERED], 0,
                  xfsm_client_get_object_path (client));
-  
+
   if (previous_id == NULL)
     {
       SmsSaveYourself (sms_conn, SmSaveLocal, False, SmInteractStyleNone, False);
@@ -933,7 +952,7 @@ xfsm_manager_start_interact (XfsmManager *manager,
   /* notify client of interact */
   SmsInteract (xfsm_client_get_sms_connection (client));
   xfsm_client_set_state (client, XFSM_CLIENT_INTERACTING);
-  
+
   /* stop save yourself timeout */
   xfsm_manager_cancel_client_save_timeout (manager, client);
 }
@@ -967,14 +986,14 @@ xfsm_manager_interact (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) == XFSM_CLIENT_INTERACTING)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) == XFSM_CLIENT_INTERACTING)
             {
-              xfsm_client_set_state (client, XFSM_CLIENT_WAITFORINTERACT);
+              xfsm_client_set_state (cl, XFSM_CLIENT_WAITFORINTERACT);
               return;
             }
         }
-  
+
       xfsm_manager_start_interact (manager, client);
     }
 }
@@ -1004,9 +1023,9 @@ xfsm_manager_interact_done (XfsmManager *manager,
       xfsm_manager_close_connection (manager, client, TRUE);
       return;
     }
-  
+
   xfsm_client_set_state (client, XFSM_CLIENT_SAVING);
-  
+
   /* Setting the cancel-shutdown field to True indicates that the user
    * has requested that the entire shutdown be cancelled. Cancel-
    * shutdown may only be True if the corresponding SaveYourself
@@ -1018,7 +1037,7 @@ xfsm_manager_interact_done (XfsmManager *manager,
     {
       /* we go into checkpoint state from here... */
       xfsm_manager_set_state (manager, XFSM_MANAGER_CHECKPOINT);
-      
+
       /* If a shutdown is in progress, the user may have the option
        * of cancelling the shutdown. If the shutdown is cancelled
        * (specified in the "Interact Done" message), the session
@@ -1029,13 +1048,13 @@ xfsm_manager_interact_done (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) != XFSM_CLIENT_WAITFORINTERACT)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) != XFSM_CLIENT_WAITFORINTERACT)
             continue;
 
           /* reset all clients that are waiting for interact */
           xfsm_client_set_state (client, XFSM_CLIENT_SAVING);
-          SmsShutdownCancelled (xfsm_client_get_sms_connection (client));
+          SmsShutdownCancelled (xfsm_client_get_sms_connection (cl));
         }
 
         g_signal_emit (manager, signals[SIG_SHUTDOWN_CANCELLED], 0);
@@ -1047,10 +1066,10 @@ xfsm_manager_interact_done (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) == XFSM_CLIENT_WAITFORINTERACT)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) == XFSM_CLIENT_WAITFORINTERACT)
             {
-              xfsm_manager_start_interact (manager, client);
+              xfsm_manager_start_interact (manager, cl);
               break;
             }
         }
@@ -1060,6 +1079,7 @@ xfsm_manager_interact_done (XfsmManager *manager,
   xfsm_manager_start_client_save_timeout (manager, client);
 }
 
+
 static void
 xfsm_manager_save_yourself_global (XfsmManager     *manager,
                                    gint             save_type,
@@ -1069,8 +1089,9 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
                                    XfsmShutdownType shutdown_type,
                                    gboolean         allow_shutdown_save)
 {
-  gboolean shutdown_save = allow_shutdown_save;
-  GList *lp;
+  gboolean  shutdown_save = allow_shutdown_save;
+  GList    *lp;
+  GError   *error = NULL;
 
   if (shutdown)
     {
@@ -1078,7 +1099,7 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
         {
           /* if we're not specifying fast shutdown, and we're ok with
            * prompting then ask the user what to do */
-          if (!shutdownDialog (manager->session_name, &manager->shutdown_type, &shutdown_save))
+          if (!xfsm_logout_dialog (manager->session_name, &manager->shutdown_type, &shutdown_save))
             return;
 
           /* |allow_shutdown_save| is ignored if we prompt the user.  i think
@@ -1095,16 +1116,9 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
       if (manager->shutdown_type == XFSM_SHUTDOWN_SUSPEND
           || manager->shutdown_type == XFSM_SHUTDOWN_HIBERNATE)
         {
-          XfsmShutdownHelper *shutdown_helper;
-          GError *error = NULL;
-
-          shutdown_helper = xfsm_shutdown_helper_spawn (&error);
-          if (shutdown_helper == NULL
-              || (!xfsm_shutdown_helper_send_command(shutdown_helper,
-                                                    manager->shutdown_type,
-                                                    &error)
-                  && (error->domain != DBUS_GERROR
-                      || error->code != DBUS_GERROR_NO_REPLY)))
+          if (!xfsm_shutdown_try_type (manager->shutdown_helper,
+                                       manager->shutdown_type,
+                                       &error))
             {
               xfce_message_dialog (NULL, _("Shutdown Failed"),
                                    GTK_STOCK_DIALOG_ERROR,
@@ -1117,36 +1131,12 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
               g_error_free (error);
             }
 
-          /* clean up and return */
-          if (G_LIKELY (shutdown_helper))
-            xfsm_shutdown_helper_destroy (shutdown_helper);
-
           /* at this point, either we failed to suspend/hibernate, or we
            * successfully suspended/hibernated, and we've been woken back
            * up, so return control to the user */
           return;
         }
     }
-#if defined(__NR_ioprio_set) && defined(HAVE_SYNC)
-  /* if we're on Linux and have ioprio_set(), we start sync()ing the
-   * disks now, and set the i/o priority to idle so we don't create
-   * a poor user experience if any apps need to interact with the user
-   * during shutdown.  if we *don't* have ioprio_set(), we sync at the
-   * end of shutdown, right before quitting.
-   */
-  if (shutdown == TRUE && fork () == 0)
-    {
-#ifdef HAVE_SETSID
-      setsid ();
-#endif
-      if(!syscall (__NR_ioprio_set, IOPRIO_WHO_PROCESS, getpid (),
-                   IOPRIO_PRIO_VALUE (IOPRIO_CLASS_IDLE, 0)))
-        {
-          sync ();
-        }
-      _exit (EXIT_SUCCESS);
-    }
-#endif
 
   if (!shutdown || shutdown_save)
     {
@@ -1154,7 +1144,7 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
                               shutdown
                               ? XFSM_MANAGER_SHUTDOWN
                               : XFSM_MANAGER_CHECKPOINT);
-      
+
       /* handle legacy applications first! */
       xfsm_legacy_perform_session_save ();
 
@@ -1164,14 +1154,13 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
         {
           XfsmClient *client = lp->data;
           XfsmProperties *properties = xfsm_client_get_properties (client);
+          const gchar *program;
 
           /* xterm's session management is broken, so we won't
            * send a SAVE YOURSELF to xterms */
-          if (properties->program != NULL
-              && strcasecmp (properties->program, "xterm") == 0)
-            {
-              continue;
-            }
+          program = xfsm_properties_get_string (properties, SmProgram);
+          if (program != NULL && strcasecmp (program, "xterm") == 0)
+            continue;
 
           if (xfsm_client_get_state (client) != XFSM_CLIENT_SAVINGLOCAL)
             {
@@ -1181,7 +1170,7 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
 
           xfsm_client_set_state (client, XFSM_CLIENT_SAVING);
           xfsm_manager_start_client_save_timeout (manager, client);
-        } 
+        }
     }
   else
     {
@@ -1202,6 +1191,8 @@ xfsm_manager_save_yourself (XfsmManager *manager,
 {
   if (G_UNLIKELY (xfsm_client_get_state (client) != XFSM_CLIENT_IDLE))
     {
+
+
       xfsm_verbose ("Client Id = %s, requested SAVE YOURSELF, but client is not in IDLE mode.\n"
                     "   Client will be nuked now.\n\n",
                     xfsm_client_get_id (client));
@@ -1216,7 +1207,7 @@ xfsm_manager_save_yourself (XfsmManager *manager,
       xfsm_manager_close_connection (manager, client, TRUE);
       return;
     }
-  
+
   if (!global)
     {
       /* client requests a local checkpoint. We slightly ignore
@@ -1263,7 +1254,7 @@ xfsm_manager_save_yourself_done (XfsmManager *manager,
       xfsm_verbose ("Client Id = %s send SAVE YOURSELF DONE, while not being "
                     "in save mode. Prepare to be nuked!\n",
                     xfsm_client_get_id (client));
-      
+
       xfsm_manager_close_connection (manager, client, TRUE);
     }
 
@@ -1298,7 +1289,7 @@ xfsm_manager_close_connection (XfsmManager *manager,
 {
   IceConn ice_conn;
   GList *lp;
-  
+
   xfsm_client_set_state (client, XFSM_CLIENT_DISCONNECTED);
   xfsm_manager_cancel_client_save_timeout (manager, client);
 
@@ -1310,18 +1301,18 @@ xfsm_manager_close_connection (XfsmManager *manager,
       IceSetShutdownNegotiation (ice_conn, False);
       IceCloseConnection (ice_conn);
     }
-  
+
   if (manager->state == XFSM_MANAGER_SHUTDOWNPHASE2)
     {
       for (lp = g_queue_peek_nth_link (manager->running_clients, 0);
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) != XFSM_CLIENT_DISCONNECTED)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) != XFSM_CLIENT_DISCONNECTED)
             return;
         }
-      
+
       /* all clients finished the DIE phase in time */
       if (manager->die_timeout_id)
         {
@@ -1335,7 +1326,7 @@ xfsm_manager_close_connection (XfsmManager *manager,
       xfsm_verbose ("Client Id = %s, closed connection in checkpoint state\n"
                     "   Session manager will show NO MERCY\n\n",
                     xfsm_client_get_id (client));
-      
+
       /* stupid client disconnected in CheckPoint state, prepare to be nuked! */
       g_queue_remove (manager->running_clients, client);
       g_object_unref (client);
@@ -1344,7 +1335,7 @@ xfsm_manager_close_connection (XfsmManager *manager,
   else
     {
       XfsmProperties *properties = xfsm_client_steal_properties (client);
-      
+
       if (properties != NULL)
         {
           if (xfsm_properties_check (properties))
@@ -1432,16 +1423,22 @@ xfsm_manager_perform_shutdown (XfsmManager *manager)
        lp = lp->next)
     {
       XfsmProperties *properties = lp->data;
+      gint            restart_style_hint;
+      gchar         **shutdown_command;
 
-      if (properties->restart_style_hint == SmRestartAnyway
-          && properties->shutdown_command != NULL)
+      restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                      SmRestartStyleHint,
+                                                      SmRestartIfRunning);
+      shutdown_command = xfsm_properties_get_strv (properties, SmShutdownCommand);
+
+      if (restart_style_hint == SmRestartAnyway && shutdown_command != NULL)
         {
           xfsm_verbose ("Client Id = %s, quit already, running shutdown command.\n\n",
                         properties->client_id);
 
-          g_spawn_sync (properties->current_directory,
-                        properties->shutdown_command,
-                        properties->environment,
+          g_spawn_sync (xfsm_properties_get_string (properties, SmCurrentDirectory),
+                        shutdown_command,
+                        xfsm_properties_get_strv (properties, SmEnvironment),
                         G_SPAWN_SEARCH_PATH,
                         NULL, NULL,
                         NULL, NULL,
@@ -1477,7 +1474,7 @@ xfsm_manager_check_clients_saving (XfsmManager *manager)
             break;
         }
     }
-  
+
   return FALSE;
 }
 
@@ -1487,13 +1484,13 @@ xfsm_manager_maybe_enter_phase2 (XfsmManager *manager)
 {
   gboolean entered_phase2 = FALSE;
   GList *lp;
-  
+
   for (lp = g_queue_peek_nth_link (manager->running_clients, 0);
        lp;
        lp = lp->next)
-    { 
+    {
       XfsmClient *client = lp->data;
-      
+
       if (xfsm_client_get_state (client) == XFSM_CLIENT_WAITFORPHASE2)
         {
           entered_phase2 = TRUE;
@@ -1520,12 +1517,12 @@ xfsm_manager_complete_saveyourself (XfsmManager *manager)
    */
   if (xfsm_manager_check_clients_saving (manager) || xfsm_manager_maybe_enter_phase2 (manager))
     return;
-  
+
   xfsm_verbose ("Manager finished SAVE YOURSELF, session data will be stored now.\n\n");
-  
+
   /* all clients done, store session data */
   xfsm_manager_store_session (manager);
-  
+
   if (manager->state == XFSM_MANAGER_CHECKPOINT)
     {
       /* reset all clients to idle state */
@@ -1551,16 +1548,16 @@ static gboolean
 xfsm_manager_save_timeout (gpointer user_data)
 {
   XfsmSaveTimeoutData *stdata = user_data;
-  
+
   xfsm_verbose ("Client id = %s, received SAVE TIMEOUT\n"
                 "   Client will be disconnected now.\n\n",
                 xfsm_client_get_id (stdata->client));
 
-  xfsm_manager_close_connection (stdata->manager, stdata->client, TRUE);
-
   /* returning FALSE below will free the data */
   g_object_steal_data (G_OBJECT (stdata->client), "--save-timeout-id");
-  
+
+  xfsm_manager_close_connection (stdata->manager, stdata->client, TRUE);
+
   return FALSE;
 }
 
@@ -1654,12 +1651,16 @@ xfsm_manager_store_session (XfsmManager *manager)
     {
       XfsmClient     *client     = lp->data;
       XfsmProperties *properties = xfsm_client_get_properties (client);
+      gint            restart_style_hint;
 
       if (properties == NULL || !xfsm_properties_check (xfsm_client_get_properties (client)))
         continue;
-      if (properties->restart_style_hint == SmRestartNever)
+      restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                      SmRestartStyleHint,
+                                                      SmRestartIfRunning);
+      if (restart_style_hint == SmRestartNever)
         continue;
-      
+
       g_snprintf (prefix, 64, "Client%d_", count);
       xfsm_properties_store (xfsm_client_get_properties (client), rc, prefix);
       ++count;
@@ -1676,7 +1677,7 @@ xfsm_manager_store_session (XfsmManager *manager)
     {
       screen = wnck_screen_get (n);
       wnck_screen_force_update (screen);
-      
+
       workspace = wnck_screen_get_active_workspace (screen);
       m = wnck_workspace_get_number (workspace);
 
@@ -1755,6 +1756,12 @@ xfsm_manager_get_compat_startup (XfsmManager          *manager,
 }
 
 
+gboolean
+xfsm_manager_get_start_at (XfsmManager *manager)
+{
+  return manager->start_at;
+}
+
 
 /*
  * dbus server impl
@@ -1778,14 +1785,36 @@ static gboolean xfsm_manager_dbus_get_state (XfsmManager *manager,
 static gboolean xfsm_manager_dbus_checkpoint (XfsmManager *manager,
                                               const gchar *session_name,
                                               GError     **error);
+static gboolean xfsm_manager_dbus_logout (XfsmManager *manager,
+                                          gboolean     show_dialog,
+                                          gboolean     allow_save,
+                                          GError     **error);
 static gboolean xfsm_manager_dbus_shutdown (XfsmManager *manager,
-                                            guint        type,
                                             gboolean     allow_save,
                                             GError     **error);
+static gboolean xfsm_manager_dbus_can_shutdown (XfsmManager *manager,
+                                                gboolean    *can_shutdown,
+                                                GError     **error);
+static gboolean xfsm_manager_dbus_restart (XfsmManager *manager,
+                                           gboolean     allow_save,
+                                           GError     **error);
+static gboolean xfsm_manager_dbus_can_restart (XfsmManager *manager,
+                                               gboolean    *can_restart,
+                                               GError     **error);
+static gboolean xfsm_manager_dbus_suspend (XfsmManager *manager,
+                                           GError     **error);
+static gboolean xfsm_manager_dbus_can_suspend (XfsmManager *manager,
+                                               gboolean    *can_suspend,
+                                               GError     **error);
+static gboolean xfsm_manager_dbus_hibernate (XfsmManager *manager,
+                                             GError     **error);
+static gboolean xfsm_manager_dbus_can_hibernate (XfsmManager *manager,
+                                                 gboolean    *can_hibernate,
+                                                 GError     **error);
 
 
 /* eader needs the above fwd decls */
-#include "xfsm-manager-dbus.h"
+#include <xfce4-session/xfsm-manager-dbus.h>
 
 
 static void
@@ -1800,10 +1829,11 @@ static void
 xfsm_manager_dbus_init (XfsmManager *manager)
 {
   GError *error = NULL;
+  DBusConnection *connection;
 
-  manager->dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  manager->session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 
-  if (G_UNLIKELY (!manager->dbus_conn))
+  if (G_UNLIKELY (!manager->session_bus))
     {
       g_critical ("Unable to contact D-Bus session bus: %s", error ? error->message : "Unknown error");
       if (error)
@@ -1811,11 +1841,14 @@ xfsm_manager_dbus_init (XfsmManager *manager)
       return;
     }
 
-  dbus_g_connection_register_g_object (manager->dbus_conn,
+  connection = dbus_g_connection_get_connection (manager->session_bus);
+  dbus_connection_set_exit_on_disconnect (connection, FALSE);
+
+  dbus_g_connection_register_g_object (manager->session_bus,
                                        "/org/xfce/SessionManager",
                                        G_OBJECT (manager));
 
-  dbus_connection_add_filter (dbus_g_connection_get_connection (manager->dbus_conn),
+  dbus_connection_add_filter (dbus_g_connection_get_connection (manager->session_bus),
                               xfsm_manager_watch_dbus_disconnect,
                               manager, NULL);
 }
@@ -1824,13 +1857,13 @@ xfsm_manager_dbus_init (XfsmManager *manager)
 static void
 xfsm_manager_dbus_cleanup (XfsmManager *manager)
 {
-  if (G_LIKELY (manager->dbus_conn))
+  if (G_LIKELY (manager->session_bus))
     {
-      dbus_connection_remove_filter (dbus_g_connection_get_connection (manager->dbus_conn),
+      dbus_connection_remove_filter (dbus_g_connection_get_connection (manager->session_bus),
                                      xfsm_manager_watch_dbus_disconnect,
                                      manager);
-      dbus_g_connection_unref (manager->dbus_conn);
-      manager->dbus_conn = NULL;
+      dbus_g_connection_unref (manager->session_bus);
+      manager->session_bus = NULL;
     }
 }
 
@@ -1951,10 +1984,10 @@ xfsm_manager_dbus_shutdown_idled (gpointer data)
 
 
 static gboolean
-xfsm_manager_dbus_shutdown (XfsmManager *manager,
-                            guint        type,
-                            gboolean     allow_save,
-                            GError     **error)
+xfsm_manager_save_yourself_dbus (XfsmManager       *manager,
+                                 XfsmShutdownType   type,
+                                 gboolean           allow_save,
+                                 GError           **error)
 {
   ShutdownIdleData *idata;
 
@@ -1965,13 +1998,6 @@ xfsm_manager_dbus_shutdown (XfsmManager *manager,
       return FALSE;
     }
 
-  if (type > XFSM_SHUTDOWN_HIBERNATE)
-    {
-      g_set_error (error, XFSM_ERROR, XFSM_ERROR_BAD_VALUE,
-                   _("Invalid shutdown type \"%u\""), type);
-      return FALSE;
-    }
-
   idata = g_new (ShutdownIdleData, 1);
   idata->manager = manager;
   idata->type = type;
@@ -1981,3 +2007,118 @@ xfsm_manager_dbus_shutdown (XfsmManager *manager,
 
   return TRUE;
 }
+
+
+static gboolean
+xfsm_manager_dbus_logout (XfsmManager *manager,
+                          gboolean     show_dialog,
+                          gboolean     allow_save,
+                          GError     **error)
+{
+  XfsmShutdownType type;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+
+  type = show_dialog ? XFSM_SHUTDOWN_ASK : XFSM_SHUTDOWN_LOGOUT;
+  return xfsm_manager_save_yourself_dbus (manager, type,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_shutdown (XfsmManager *manager,
+                            gboolean     allow_save,
+                            GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_manager_save_yourself_dbus (manager, XFSM_SHUTDOWN_SHUTDOWN,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_shutdown (XfsmManager *manager,
+                                gboolean    *can_shutdown,
+                                GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_can_shutdown (manager->shutdown_helper,
+                                     can_shutdown, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_restart (XfsmManager *manager,
+                           gboolean     allow_save,
+                           GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_manager_save_yourself_dbus (manager, XFSM_SHUTDOWN_RESTART,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_restart (XfsmManager *manager,
+                               gboolean    *can_restart,
+                               GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_can_restart (manager->shutdown_helper,
+                                    can_restart, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_suspend (XfsmManager *manager,
+                           GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_try_suspend (manager->shutdown_helper, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_suspend (XfsmManager *manager,
+                               gboolean    *can_suspend,
+                               GError     **error)
+{
+  gboolean retval;
+  gboolean auth_suspend;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  retval = xfsm_shutdown_can_suspend (manager->shutdown_helper,
+                                      can_suspend, &auth_suspend, error);
+
+  if (!auth_suspend)
+    *can_suspend = FALSE;
+
+  return retval;
+}
+
+static gboolean
+xfsm_manager_dbus_hibernate (XfsmManager *manager,
+                             GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_try_hibernate (manager->shutdown_helper, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_hibernate (XfsmManager *manager,
+                                 gboolean    *can_hibernate,
+                                 GError     **error)
+{
+  gboolean retval;
+  gboolean auth_hibernate;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  retval = xfsm_shutdown_can_hibernate (manager->shutdown_helper,
+                                        can_hibernate, &auth_hibernate, error);
+
+  if (!auth_hibernate)
+    *can_hibernate = FALSE;
+
+  return retval;
+}
