@@ -8,16 +8,16 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,7 +30,9 @@
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -43,10 +45,16 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 
 #include <glib/gstdio.h>
-
-#include <libxfcegui4/libxfcegui4.h>
+#include <gdk/gdkx.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-util.h>
 
@@ -56,7 +64,7 @@
 #include <xfce4-session/xfsm-manager.h>
 #include <xfce4-session/xfsm-splash-screen.h>
 
-#include "xfsm-startup.h"
+#include <xfce4-session/xfsm-startup.h>
 
 
 typedef struct
@@ -65,10 +73,6 @@ typedef struct
   XfsmProperties *properties;
 } XfsmStartupData;
 
-
-/*
-   Prototypes
- */
 static void     xfsm_startup_failsafe                (XfsmManager *manager);
 
 static gboolean xfsm_startup_session_next_prio_group (XfsmManager *manager);
@@ -82,10 +86,238 @@ static gboolean xfsm_startup_timeout                 (gpointer     data);
 static void     xfsm_startup_handle_failed_startup   (XfsmProperties *properties,
                                                       XfsmManager    *manager);
 
+
+static gchar *running_sshagent = NULL;
+
+
+
+static void
+xfsm_startup_init_sshagent (const gchar *cmd,
+                            const gchar *agent)
+{
+  gchar     *cmdoutput = NULL;
+  GError    *error = NULL;
+  gchar    **lines;
+  guint      i;
+  gchar     *p, *t;
+  gchar     *variable, *value;
+
+  if (g_spawn_command_line_sync (cmd, &cmdoutput, NULL, NULL, &error))
+    {
+      if (G_UNLIKELY (cmdoutput == NULL))
+        {
+          g_message ("%s returned no variables to stdout", agent);
+          return;
+        }
+
+      lines = g_strsplit (cmdoutput, "\n", -1);
+      g_assert (lines != NULL);
+      for (i = 0; lines[i] != NULL; i++)
+        {
+          p = strchr (lines[i], '=');
+          if (G_UNLIKELY (p == NULL))
+            continue;
+          t = strchr (p + 1, ';');
+          if (G_UNLIKELY (t == NULL))
+            continue;
+
+          variable = g_strndup (lines[i], p - lines[i]);
+          value = g_strndup (p + 1, t - p - 1);
+
+          g_setenv (variable, value, TRUE);
+
+          g_free (variable);
+          g_free (value);
+        }
+      g_strfreev (lines);
+
+      /* keep this around for shutdown */
+      running_sshagent = g_strdup (agent);
+    }
+  else
+    {
+      g_warning ("Failed to spawn %s: %s", agent, error->message);
+      g_error_free (error);
+    }
+
+  g_free (cmdoutput);
+}
+
+
 void
 xfsm_startup_init (XfconfChannel *channel)
 {
-  /* nothing to be done here, currently */
+  gchar       *agent;
+  gchar       *path = NULL;
+  gchar       *envfile;
+  gchar       *cmd;
+  const gchar *ssh_agent_pid;
+  pid_t        pid;
+  gboolean     gnome_keyring_found;
+
+  if (xfconf_channel_get_bool (channel, "/startup/ssh-agent/enabled", TRUE))
+    {
+      /* if GNOME compatibility is enabled and gnome-keyring-daemon
+       * is found, skip the gpg/ssh agent startup and wait for
+       * gnome-keyring, which is probably what the user wants */
+      if (xfconf_channel_get_bool (channel, "/compat/LaunchGNOME", FALSE))
+        {
+          cmd = g_find_program_in_path ("gnome-keyring-daemon");
+          gnome_keyring_found = (cmd != NULL);
+          g_free (cmd);
+
+          if (gnome_keyring_found)
+            {
+              g_print ("xfce4-session: %s\n",
+                       "GNOME compatibility is enabled and gnome-keyring-daemon is "
+                       "found on the system. Skipping gpg/ssh-agent startup.");
+              return;
+            }
+        }
+
+      agent = xfconf_channel_get_string (channel, "/startup/ssh-agent/type", NULL);
+      if (g_strcmp0 (agent, "gpg-agent") == 0
+          || g_strcmp0 (agent, "ssh-agent") == 0)
+        {
+          path = g_find_program_in_path (agent);
+        }
+      else if (agent == NULL)
+        {
+          /* lookup gpg- or ssh-agent */
+          path = g_find_program_in_path ("gpg-agent");
+          if (G_UNLIKELY (path != NULL))
+            {
+              agent = g_strdup ("gpg-agent");
+            }
+          else
+            {
+              path = g_find_program_in_path ("ssh-agent");
+              if (path != NULL)
+                agent = g_strdup ("ssh-agent");
+            }
+        }
+      else
+        {
+          g_message ("Unknown authentication agent \"%s\" set", agent);
+
+          /* avoid more errors */
+          g_free (agent);
+          return;
+        }
+
+      if (G_LIKELY (path != NULL))
+        {
+          ssh_agent_pid = g_getenv ("SSH_AGENT_PID");
+          if (ssh_agent_pid != NULL && *ssh_agent_pid == '\0')
+            ssh_agent_pid = NULL;
+
+          if (ssh_agent_pid != NULL)
+            {
+              /* check if the pid is still responding (ie not stale) */
+              pid = atoi (ssh_agent_pid);
+              if (pid > 0 && kill (pid, 0) != 0)
+                {
+                  g_unsetenv ("SSH_AGENT_PID");
+                  g_unsetenv ("SSH_AUTH_SOCK");
+
+                  ssh_agent_pid = NULL;
+                }
+            }
+
+          if (g_strcmp0 (agent, "gpg-agent") == 0)
+            {
+              envfile = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "gpg-agent-info", FALSE);
+
+              if (ssh_agent_pid == NULL)
+                {
+                  cmd = g_strdup_printf ("%s --sh --daemon --enable-ssh-support "
+                                         "--write-env-file '%s'", path, envfile);
+                  xfsm_startup_init_sshagent (cmd, agent);
+                  g_free (cmd);
+                }
+              else if (g_getenv ("GPG_AGENT_INFO") == NULL)
+                {
+                  g_message ("ssh-agent is already running; starting gpg-agent without ssh support");
+                  cmd = g_strdup_printf ("%s --sh --daemon --write-env-file '%s'", path, envfile);
+                  xfsm_startup_init_sshagent (cmd, agent);
+                  g_free (cmd);
+                }
+              else
+                {
+                  g_message ("%s is already running", agent);
+                }
+
+              g_free (envfile);
+            }
+          else if (g_strcmp0 (agent, "ssh-agent") == 0)
+            {
+              if (ssh_agent_pid == NULL)
+                {
+                  cmd = g_strdup_printf ("%s -s", path);
+                  xfsm_startup_init_sshagent (cmd, agent);
+                  g_free (cmd);
+                }
+              else
+                {
+                  g_message ("%s is already running", agent);
+                }
+            }
+        }
+      else
+        {
+          g_printerr ("xfce4-session: %s\n",
+                      "No gpg or ssh authentication agent found");
+        }
+
+      g_free (agent);
+      g_free (path);
+    }
+}
+
+
+void
+xfsm_startup_shutdown (void)
+{
+  gchar       *envfile;
+  const gchar *agentpid;
+  pid_t        pid;
+  gboolean     is_gpg_agent;
+
+  if (running_sshagent == NULL)
+    return;
+
+  agentpid = g_getenv ("SSH_AGENT_PID");
+  if (G_UNLIKELY (agentpid == NULL))
+    {
+      g_warning ("%s was started, but SSH_AGENT_PID is not set, nothing to kill", running_sshagent);
+    }
+  else
+    {
+      is_gpg_agent = g_strcmp0 (running_sshagent, "gpg-agent") == 0;
+
+      /* kill the process (gpg-agent uses SIGINT, ssh-agent SIGTERM) */
+      pid = atoi (agentpid);
+      if (pid < 1
+          || kill (pid, is_gpg_agent ? SIGINT : SIGTERM) != 0)
+        {
+          g_warning ("Failed to kill %s with pid %s", running_sshagent, agentpid);
+        }
+
+      /* drop the info file from gpg-agent */
+      if (is_gpg_agent)
+        {
+          envfile = xfce_resource_lookup (XFCE_RESOURCE_CACHE, "gpg-agent-info");
+          if (G_LIKELY (envfile != NULL))
+            g_unlink (envfile);
+          g_free (envfile);
+        }
+    }
+
+  /* make sure the env values are unset */
+  g_unsetenv ("SSH_AGENT_PID");
+  g_unsetenv ("SSH_AUTH_SOCK");
+
+  g_free (running_sshagent);
 }
 
 
@@ -263,7 +495,6 @@ xfsm_startup_autostart_migrate (void)
                                "Terminal=False\n"
                                "StartupNotify=False\n"
                                "Version=0.9.4\n"
-                               "Encoding=UTF-8\n"
                                "Name=%s\n",
                                entry, target_path, entry);
                       fclose (fp);
@@ -304,7 +535,8 @@ xfsm_startup_autostart_migrate (void)
 
 
 static gint
-xfsm_startup_autostart_xdg (void)
+xfsm_startup_autostart_xdg (XfsmManager *manager,
+                            gboolean     start_at_spi)
 {
   const gchar *try_exec;
   const gchar *type;
@@ -319,11 +551,19 @@ xfsm_startup_autostart_xdg (void)
   gchar      **not_show_in;
   gint         started = 0;
   gint         n, m;
+  gchar       *filename;
+  const gchar *pattern;
 
   /* migrate the old autostart location (if still present) */
   xfsm_startup_autostart_migrate ();
 
-  files = xfce_resource_match (XFCE_RESOURCE_CONFIG, "autostart/*.desktop", TRUE);
+  /* pattern for only at-spi desktop files or everything */
+  if (start_at_spi)
+    pattern = "autostart/at-spi-*.desktop";
+  else
+    pattern = "autostart/*.desktop";
+
+  files = xfce_resource_match (XFCE_RESOURCE_CONFIG, pattern, TRUE);
   for (n = 0; files[n] != NULL; ++n)
     {
       rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, files[n], TRUE);
@@ -336,37 +576,53 @@ xfsm_startup_autostart_xdg (void)
       skip = xfce_rc_read_bool_entry (rc, "Hidden", FALSE);
       if (G_LIKELY (!skip))
         {
-          /* check the OnlyShowIn setting */
-          only_show_in = xfce_rc_read_list_entry (rc, "OnlyShowIn", ";");
-          if (G_UNLIKELY (only_show_in != NULL))
+          if (xfce_rc_read_bool_entry (rc, "X-XFCE-Autostart-Override", FALSE))
             {
-              /* check if "Xfce" is specified */
-              for (m = 0, skip = TRUE; only_show_in[m] != NULL; ++m)
-                if (g_ascii_strcasecmp (only_show_in[m], "Xfce") == 0)
-                  {
-                    skip = FALSE;
-                    break;
-                  }
-
-              g_strfreev (only_show_in);
+              /* override the OnlyShowIn check */
+              skip = FALSE;
             }
           else
             {
-              /* check the NotShowIn setting */
-              not_show_in = xfce_rc_read_list_entry (rc, "NotShowIn", ";");
-              if (G_UNLIKELY (not_show_in != NULL))
+              /* check the OnlyShowIn setting */
+              only_show_in = xfce_rc_read_list_entry (rc, "OnlyShowIn", ";");
+              if (G_UNLIKELY (only_show_in != NULL))
                 {
-                  /* check if "Xfce" is not specified */
-                  for (m = 0; not_show_in[m] != NULL; ++m)
-                    if (g_ascii_strcasecmp (not_show_in[m], "Xfce") == 0)
-                      {
-                        skip = TRUE;
-                        break;
-                      }
+                  /* check if "XFCE" is specified */
+                  for (m = 0, skip = TRUE; only_show_in[m] != NULL; ++m)
+                    {
+                      if (g_ascii_strcasecmp (only_show_in[m], "XFCE") == 0)
+                        {
+                          skip = FALSE;
+                          break;
+                        }
+                    }
 
-                  g_strfreev (not_show_in);
+                  g_strfreev (only_show_in);
                 }
             }
+
+          /* check the NotShowIn setting */
+          not_show_in = xfce_rc_read_list_entry (rc, "NotShowIn", ";");
+          if (G_UNLIKELY (not_show_in != NULL))
+            {
+              /* check if "Xfce" is not specified */
+              for (m = 0; not_show_in[m] != NULL; ++m)
+                if (g_ascii_strcasecmp (not_show_in[m], "XFCE") == 0)
+                  {
+                    skip = TRUE;
+                    break;
+                  }
+
+              g_strfreev (not_show_in);
+            }
+
+          /* skip at-spi launchers if not in at-spi mode or don't skip
+           * them no matter what the OnlyShowIn key says if only
+           * launching at-spi */
+          filename = g_path_get_basename (files[n]);
+          if (g_str_has_prefix (filename, "at-spi-"))
+            skip = !start_at_spi;
+          g_free (filename);
         }
 
       /* check the "Type" key */
@@ -389,7 +645,11 @@ xfsm_startup_autostart_xdg (void)
 
           /* try to launch the command */
           xfsm_verbose ("Autostart: running command \"%s\"\n", exec);
-          if (!xfce_exec (exec, terminal, startup_notify, &error))
+          if (!xfce_spawn_command_line_on_screen (gdk_screen_get_default (),
+                                                  exec,
+                                                  terminal,
+                                                  startup_notify,
+                                                  &error))
             {
               g_warning ("Unable to launch \"%s\" (specified by %s): %s", exec, files[n], error->message);
               g_error_free (error);
@@ -412,11 +672,11 @@ xfsm_startup_autostart_xdg (void)
 
 
 static void
-xfsm_startup_autostart (void)
+xfsm_startup_autostart (XfsmManager *manager)
 {
   gint n;
 
-  n = xfsm_startup_autostart_xdg ();
+  n = xfsm_startup_autostart_xdg (manager, FALSE);
 
   if (n > 0)
     {
@@ -444,13 +704,121 @@ xfsm_startup_foreign (XfsmManager *manager)
 }
 
 
+static void
+xfsm_startup_at_set_gtk_modules (void)
+{
+  const gchar  *old;
+  gchar       **modules;
+  guint         i;
+  gboolean      found_gail = FALSE;
+  gboolean      found_atk_bridge = FALSE;
+  GString      *new;
+
+  old = g_getenv ("GTK_MODULES");
+  if (old != NULL && *old != '\0')
+    {
+      /* check which modules are already loaded */
+      modules = g_strsplit (old, ":", -1);
+      for (i = 0; modules[i] != NULL; i++)
+        {
+          if (strcmp (modules[i], "gail") == 0)
+            found_gail = TRUE;
+          else if (strcmp (modules[i], "atk-bridge") == 0)
+            found_atk_bridge = TRUE;
+        }
+      g_strfreev (modules);
+
+      if (!found_gail || !found_atk_bridge)
+        {
+          /* append modules to old value */
+          new = g_string_new (old);
+          if (!found_gail)
+            new = g_string_append (new, ":gail");
+          if (!found_atk_bridge)
+            new = g_string_append (new, ":atk-bridge");
+
+          g_setenv ("GTK_MODULES", new->str, TRUE);
+          g_string_free (new, TRUE);
+        }
+    }
+  else
+    {
+      g_setenv ("GTK_MODULES", "gail:atk-bridge", TRUE);
+    }
+}
+
+
+static gboolean
+xfsm_startup_at_spi_ior_set (void)
+{
+  Atom        AT_SPI_IOR;
+  GdkDisplay *display;
+  Atom        actual_type;
+  gint        actual_format;
+  guchar    *data = NULL;
+  gulong     nitems;
+  gulong     leftover;
+
+  display = gdk_display_get_default ();
+  AT_SPI_IOR = XInternAtom (GDK_DISPLAY_XDISPLAY (display), "AT_SPI_IOR", False);
+  XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display),
+                      XDefaultRootWindow (GDK_DISPLAY_XDISPLAY (display)),
+                      AT_SPI_IOR, 0L,
+                      (long) BUFSIZ, False,
+                      (Atom) 31, &actual_type, &actual_format,
+                      &nitems, &leftover, &data);
+
+  if (data == NULL)
+    return FALSE;
+  XFree (data);
+
+  return TRUE;
+}
+
+
+static void
+xfsm_startup_at (XfsmManager *manager)
+{
+  gint n, i;
+
+  /* start at-spi-dbus-bus and/or at-spi-registryd */
+  n = xfsm_startup_autostart_xdg (manager, TRUE);
+
+  if (n > 0)
+    {
+      if (G_LIKELY (splash_screen != NULL))
+        xfsm_splash_screen_next (splash_screen, _("Starting Assistive Technologies"));
+
+      xfsm_startup_at_set_gtk_modules ();
+
+      /* wait for 2 seconds until the at-spi registered, not very nice
+       * but required to properly start an accessible desktop */
+      for (i = 0; i < 10; i++)
+        {
+          if (xfsm_startup_at_spi_ior_set ())
+            break;
+          xfsm_verbose ("Waiting for at-spi to register...\n");
+          g_usleep (G_USEC_PER_SEC / 5);
+        }
+    }
+  else
+    {
+      g_warning ("No assistive technology service provider was started!");
+    }
+}
+
+
 void
 xfsm_startup_begin (XfsmManager *manager)
 {
+  /* start assistive technology before anything else */
+  if (xfsm_manager_get_start_at (manager))
+    xfsm_startup_at (manager);
+
   if (xfsm_manager_get_use_failsafe_mode (manager))
     {
       xfsm_startup_failsafe (manager);
-      xfsm_startup_autostart ();
+      xfsm_startup_autostart (manager);
       xfsm_manager_signal_startup_done (manager);
     }
   else
@@ -490,26 +858,31 @@ xfsm_startup_start_properties (XfsmProperties *properties,
 {
   XfsmStartupData *child_watch_data;
   XfsmStartupData *startup_timeout_data;
+  gchar          **restart_command;
   gchar          **argv;
   gint             argc;
   gint             n;
+  const gchar     *current_directory;
   GPid             pid;
 
   /* release any possible old resources related to a previous startup */
   xfsm_properties_set_default_child_watch (properties);
 
   /* generate the argument vector for the application (expanding variables) */
-  argc = g_strv_length (properties->restart_command);
+  restart_command = xfsm_properties_get_strv (properties, SmRestartCommand);
+  argc = g_strv_length (restart_command);
   argv = g_new (gchar *, argc + 1);
   for (n = 0; n < argc; ++n)
-    argv[n] = xfce_expand_variables (properties->restart_command[n], NULL);
+    argv[n] = xfce_expand_variables (restart_command[n], NULL);
   argv[n] = NULL;
 
+  current_directory = xfsm_properties_get_string (properties, SmCurrentDirectory);
+
   /* fork a new process for the application */
 #ifdef HAVE_VFORK
   /* vfork() doesn't allow you to do anything but call exec*() or _exit(),
    * so if we need to set the working directory, we can't use vfork() */
-  if (properties->current_directory == NULL)
+  if (current_directory == NULL)
     pid = vfork ();
   else
 #endif
@@ -519,10 +892,10 @@ xfsm_startup_start_properties (XfsmProperties *properties,
   if (pid == 0)
     {
       /* execute the application here */
-      if (properties->current_directory)
+      if (current_directory)
         {
-          if (chdir (properties->current_directory))
-            g_warning ("Unable to chdir to \"%s\": %s", properties->current_directory, strerror (errno));
+          if (chdir (current_directory))
+            g_warning ("Unable to chdir to \"%s\": %s", current_directory, strerror (errno));
         }
       execvp (argv[0], argv);
       _exit (127);
@@ -582,7 +955,7 @@ xfsm_startup_session_continue (XfsmManager *manager)
       /* we failed to start anything, and we don't have anything else,
        * to start, so just move on to the autostart items and signal
        * the manager that we're finished */
-      xfsm_startup_autostart ();
+      xfsm_startup_autostart (manager);
       xfsm_manager_signal_startup_done (manager);
     }
 }
@@ -602,14 +975,14 @@ xfsm_startup_session_next_prio_group (XfsmManager *manager)
   if (properties == NULL)
     return FALSE;
 
-  cur_prio_group = properties->priority;
+  cur_prio_group = xfsm_properties_get_uchar (properties, GsmPriority, 50);
 
   xfsm_verbose ("Starting apps in prio group %d\n", cur_prio_group);
 
   while ((properties = g_queue_pop_head (pending_properties)))
     {
       /* quit if we've hit all the clients in the current prio group */
-      if (properties->priority != cur_prio_group)
+      if (xfsm_properties_get_uchar (properties, GsmPriority, 50) != cur_prio_group)
         {
           /* we're not starting this one yet; put it back */
           g_queue_push_head (pending_properties, properties);
@@ -619,8 +992,34 @@ xfsm_startup_session_next_prio_group (XfsmManager *manager)
       /* FIXME: splash */
       if (G_LIKELY (splash_screen != NULL))
         {
-          xfsm_splash_screen_next (splash_screen,
-                                   figure_app_name (properties->program));
+          const gchar *app_name = NULL;
+          const gchar *desktop_file;
+          XfceRc      *rcfile = NULL;
+
+          desktop_file = xfsm_properties_get_string (properties, GsmDesktopFile);
+
+          if (desktop_file)
+            {
+              rcfile = xfce_rc_simple_open (desktop_file, TRUE);
+              if (rcfile)
+                {
+                  xfce_rc_set_group (rcfile, "Desktop Entry");
+                  app_name = xfce_rc_read_entry (rcfile, "Name", NULL);
+                }
+            }
+
+          if (!app_name)
+            app_name = figure_app_name (xfsm_properties_get_string (properties,
+                                                                    SmProgram));
+
+          xfsm_splash_screen_next (splash_screen, app_name);
+
+          if (rcfile)
+            {
+              /* delay closing because app_name belongs to the rcfile
+               * if we found it in the file */
+              xfce_rc_close (rcfile);
+            }
         }
 
       if (G_LIKELY (xfsm_startup_start_properties (properties, manager)))
@@ -659,6 +1058,9 @@ xfsm_startup_child_watch (GPid     pid,
   xfsm_verbose ("Client Id = %s, PID %d exited with status %d\n",
                 cwdata->properties->client_id, (gint)pid, status);
 
+  cwdata->properties->child_watch_id = 0;
+  cwdata->properties->pid = -1;
+
   starting_properties = xfsm_manager_get_queue (cwdata->manager, XFSM_MANAGER_QUEUE_STARTING_PROPS);
   if (g_queue_find (starting_properties, cwdata->properties) != NULL)
     {
@@ -667,8 +1069,9 @@ xfsm_startup_child_watch (GPid     pid,
       xfsm_startup_handle_failed_startup (cwdata->properties, cwdata->manager);
     }
 
-  cwdata->properties->child_watch_id = 0;
-  cwdata->properties->pid = -1;
+  /* NOTE: cwdata->properties could have been freed by
+   * xfsm_startup_handle_failed_startup() above, so don't try to access
+   * any of its members here. */
 
   g_spawn_close_pid (pid);
 }
